# -*- coding: iso8859-1 -*-
#=========================================================================================
#                                                                         
#   This program is a cardware software; you can redistribute it and/or modify
#   it under the terms of the Creative Common Licence defined on :
#   http://creativecommons.org/licenses/by-nc-sa/2.0/fr/
#
#   (CC) 2005 by Ronan BOISARD <ronan_boisard@yahoo.com>
#
#=========================================================================================
#                                                                         
#   Ce programme est un cardware, vous pouvez l'utiliser, le modifier et le redistribuer
#   sous les condition dfinies dans la licence Creative Common dont les condition sont
#   dcritent sur
#   http://creativecommons.org/licenses/by-nc-sa/2.0/fr/
#
#   (CC) 2005 by Ronan BOISARD <ronan_boisard@yahoo.com>
#
#=========================================================================================

#=========================================================================================
"""
    Classe charge des communications avec Mysql
    Egalement utilise pour un acces direct a Mysql a travers l'application
"""
#-----------------------------------------------------------------------------------------
__version__ ='0.4'
__author__='Ronan BOISARD <ronan_boisard@yahoo.com>'
#=========================================================================================

from Tkinter import *
import tkFileDialog
import MySQLdb,re,Pmw
import tools,dialog

class Mysql:
    """
    Classe charge des communications avec Mysql
    Egalement utilise pour un acces direct a Mysql a travers l'application
    """

    def __init__(self,config,top=None):
        """
        initialise les objets de la classe avec les parametres de connection (user,
        host, pwd et db)
        """
        if top is None:
            self.top=config.top
        else:
            self.top=top
        self.fullwindow=False           # fenetre pleine cran ou onlget            
        self.user=config.user.get()
        self.host=config.host.get()
        self.pwd=config.pwd.get()
        self.db=config.db.get()

#-----------------------------------------------------------------------------------------
# Methodes utilises par Bio et Photo
#-----------------------------------------------------------------------------------------

    def connect(self):
        """
        Cr une connection a la base de donne
        """
        self.connexion=MySQLdb.connect(host=self.host,user=self.user,passwd=self.pwd,
                                       db=self.db)
        self.cursor=self.connexion.cursor()

    def deconnect(self):
        """
        Ferme la connection active
        """
        self.connexion.close()

    def IsOn(self):
        """
        Retourne un boolen, True si MySQL est accesible, False si non
        """
        try:
            conn=MySQLdb.connect()
            conn.close()
            return True
        except:
            return False        

    def DbIsOn(self):
        """
        Retourne un boolen, True si la base de donne est accesible, False si non
        """
        try:
            self.connect()
            self.deconnect()
            return True
        except:
            return False

    def DbHaveTable(self):
        """
        Retourne un boolen, True si la base de donne a les bonnes tables
        d'initialises, False si non
        """
        self.connect()
        self.cursor.execute("SHOW TABLES")
        res=self.cursor.fetchall()
        self.deconnect()
        tables=['bio','bio_photo','media','photo']
        foundtable=[]
        for item in res:
            foundtable.append(item[0])
        for item in tables:
            if item not in foundtable:
                return False
        return True


    def select(self,table,champs,condition=None):
        """
        execute une requete de la forme:
            SELECT 'champ' FROM 'table' WHERE 'condition
        et retourne le resultat sous forme de liste
        """
        self.connect()
        requete='SELECT '
        for item in champs:
            requete+=item+' , '
        requete=requete[:-2]
        requete+=' FROM '+table
        if condition is not None:
            requete+=' WHERE '+condition
        res=self.execute(requete)
        self.deconnect()
        return res

    def ajouter(self,table,elements):
        """
        Ajoute les elements 'elements' dans la table 'table'
            INSERT INTO 'table' SET 'elements'
        """
        self.connect()
        requete="INSERT INTO "+table+" SET "
        for item in elements:
            requete+=str(item)+" , "
        requete=requete[:-2]
        res=self.execute(requete)
        if table!='bio_photo':
            id=self.cursor.insert_id()
        else:
            id=None
        self.deconnect()
        return res,id

    def modifier(self,table,elements,id):
        """
        Modifie les elements 'elements' dans la table 'table'
            Update 'table' SET 'elements Where ID=id
        """
        self.connect()
        requete="UPDATE "+table+" SET "
        for item in elements:
            requete+=str(item)+" , "
        requete=requete[:-2]
        requete+=" WHERE ID='"+id+"'"
        res=self.execute(requete)
        self.deconnect()
        return res

    def execute(self,requete):
        """
        Execute une requete
        """
        try:
            self.cursor.execute(requete)
            return self.cursor.fetchall()
        except MySQLdb.Error, e:
            msg="Erreur %d: %s" % (e.args[0], e.args[1])
            msg=re.sub(';','\n',msg)
            dialog.Errormsg(self.top,msg).show()
            return 'error'

    def ElementExist(self,table,condition):
        """
        retourn un booleen, True si au moins un element dfinit par la condition est
        prsent dans la table
        """
        self.connect()
        requete='SELECT ID FROM '+table
        requete+=' WHERE '+condition
        res=self.execute(requete)
        if self.cursor.rowcount!=0:
            exist=True
            id=str(res[0][0])
        else:
            exist=False
            id=None
        self.deconnect()
        return exist,id

    def SupprimerBio(self,id):
        """
        Supprime la fiche bio 'id'
        """
        self.connect()
        requete="DELETE FROM bio WHERE ID="+str(id)
        res=self.execute(requete)
        if res!='error':
            requete="DELETE FROM bio_photo WHERE IDbio="+str(id)
            res=self.execute(requete)
        self.deconnect()
        return res
    
    def SupprimerMedia(self,id):
        """
        Supprime la fiche media 'id'
        """
        self.connect()
        requete="DELETE FROM media WHERE ID="+str(id)
        res=self.execute(requete)
        self.deconnect()
        return res    
    
    def SupprimerPhoto(self,id):
        """
        Supprime la fiche photo 'id'
        """
        self.connect()
        requete="DELETE FROM photo WHERE ID="+str(id)
        res=self.execute(requete)
        if res!='error':
            requete="DELETE FROM bio_photo WHERE IDphoto="+str(id)
            res=self.execute(requete)
        self.deconnect()
        return res


    def Bio2Photo(self,listID):
        """
        retourne la liste des id de photo correspondant  l'id bio donne
        """
        self.connect()
        listPhoto=[]
        for id in listID:
            res=self.select('bio_photo',['IDphoto'],"IDbio='"+str(id)+"'")
            for item in res:
                if item not in listPhoto:
                    listPhoto.append(str(item[0]))
        self.deconnect()
        return listPhoto

    def Photo2Bio(self,listID):
        """
        retourne la liste des id de bio corresponadnt a l'id photo donne
        """
        self.connect()
        listBio=[]
        for id in listID:
            res=self.select('bio_photo',['IDbio'],"IDphoto='"+str(id)+"'")
            for item in res:
                if item not in listBio:
                    listBio.append(str(item[0]))
        self.deconnect()
        return listBio

    def SupprimerLien(self,idbio,idphoto):
        """
        Supprime une entre de la table bio_photo
        """
        self.connect()
        requete="DELETE FROM bio_photo WHERE IDbio="+str(idbio)+" AND IDphoto="+str(idphoto)
        res=self.execute(requete)
        self.deconnect()
        return res

    def SupprimerLienPhoto(self,idphoto):
        """
        Supprime tous les liens bio vers une photo
        """
        self.connect()
        requete="DELETE FROM bio_photo WHERE IDphoto="+str(idphoto)
        res=self.execute(requete)
        self.deconnect()
        return res

    def getNoms(self,N1,N2,N3,id=None):
        """
        retourne deux listes, avec les noms communs et les nomns latins
        """
        listcommun=[]
        listcommun.append('')
        listlatin=[]
        listlatin.append('')
        requete="SELECT commun, latin FROM bio WHERE 1"
        if id is not None:
            requete+=' AND ID='+str(id)
        if N1 not in ['','Tous']:
            requete+=" AND embranchement='"+tools.nettoyer(N1)+"'"
        if N2 not in ['','Tous']:
            requete+=" AND classe='"+tools.nettoyer(N2)+"'"
        if N3 not in ['','Tous']:
            requete+=" AND espece='"+tools.nettoyer(N3)+"'"
        requete+=' ORDER BY commun'
        self.connect()
        res=self.execute(requete)
        self.deconnect()
        if res!='error':
            for item in res:
                listcommun.append(str(item[0]))
                listlatin.append(str(item[1]))
            return listcommun,listlatin
        else:
            return res

    def GetID(self,table,element):
        """
        retourne l'ID de l'lment element de la table table
        """
        requete='SELECT ID FROM '+str(table)+' WHERE '+str(element)
        self.connect()
        res=self.execute(requete)
        self.deconnect()
        if res!='error':
            return res[0][0]
        else:
            return res

    def LoadFile(self,file):
        self.connect()
        fid=open(file,'r')
        for line in fid.readlines():
#            requete=unicode(line,'latin-1')
            requete=line[:-1]
            try:
                res=self.execute(requete)
            except:
                pass
            if res=='error':
                exit
        if res!='error':
            dialog.Info(self.top, msg="La base MySQL a t charge").show()
        else:
            dialog.Info(self.top, msg="Fichier corrompu\n impossible de charger la base").show()
        self.deconnect()

    def SaveFile(self,file):
        self.connect()
        sqlfile = open(file, "w")
        self.dump_sql(sqlfile)
        dialog.Info(self.top, msg="La base MySSL a t sauvegarde").show()
        self.deconnect()

        
    def dump_sql(self,file):
        for table, create_def in self.get_structure_sql().iteritems():
            file.write('DROP TABLE '+str(table)+";\n")
            file.write(str(create_def)+";\n")
            self.dump_data_sql(file,table)

    def get_structure_sql(self):
	self.cursor.execute("SET OPTION SQL_QUOTE_SHOW_CREATE=1")
	schemas = {}
	for table in self.get_table_list():
		schemas[table] = re.sub('\\n','',self.get_table_schema(table))
	return schemas
    
    def get_table_list(self):
	self.cursor.execute("SHOW TABLES")
	return [ table for table, in self.cursor ]

    def get_table_schema(self, table):
	self.cursor.execute("SHOW CREATE TABLE %s" % (table))
	return self.cursor.fetchone()[1]

    def get_column_names(self,table):
	self.cursor.execute("DESCRIBE %s" % (table))
	return [ row[0] for row in self.cursor ]
    
    def dump_data_sql(self,file,table,condition=None):
	colnames = self.get_column_names(table)
	colnames_sql = ', '.join(colnames)
	if condition is None:
            count = self.cursor.execute("SELECT %s FROM %s" % (colnames_sql, table))
        else:
            count = self.cursor.execute("SELECT %s FROM %s WHERE %s" % (colnames_sql,table,condition))
	if count == 0:
		return
	file.write( "REPLACE INTO %s(%s) VALUES" % (table, colnames_sql))
	count -= 1
	for index, row in enumerate(self.cursor):
	    row_sql = "   (%s)" % (', '.join(self.connexion.escape(row)))
	    if index < count:
		file.write( row_sql + ",")
	    else:
		file.write( row_sql + ";\n")
    

#-----------------------------------------------------------------------------------------
# Specificits pour la page d'acces direct a MySQL
#-----------------------------------------------------------------------------------------
    def MiseEnPage(self):
        self.win=Pmw.PanedWidget(self.top,orient='vertical')
        self.win.pack(side="top",fill="both",expand="yes")
        self.win.add('requete',min=0,max=270)
        self.win.add('resultat',min=0)

    def Menu(self):
        """
        Affiche le menu de la page MySQL
        """
        self.menu=Frame(self.top)
        self.menu.pack(side='top',fill='x',expand='no')
        b = Button(self.menu,relief="flat", text='Fermer',compound='left',
			image=self.gui.pixmap['fermer'],
			command=self.Fermer,overrelief="groove",
			anchor='n')
	b.pack(side="right", padx=20, pady=0)
        b = Button(self.menu,relief="flat", text='Vider',compound='left',
			image=self.gui.pixmap['vider'],
			command=self.Vider,overrelief="groove",
			anchor='n')
	b.pack(side="right", padx=20, pady=0)	
	b = Button(self.menu,relief="flat", text='Executer',compound='left',
			image=self.gui.pixmap['sauver'],
			command=self.Executer,overrelief="groove",
			anchor='n')
	b.pack(side="right", padx=20, pady=0)        

    def Afficher(self,gui):
        """
        Gere la page destine a l'acces direct a Mysql a travers l'application
        """
        self.gui=gui
        self.fichier=StringVar()
        self.label=None
        self.text=None
        self.Menu()
        self.MiseEnPage()
        self.frame=Pmw.ScrolledFrame(self.win.pane("requete"),)
        self.frame.pack(side='top',expand='yes',fill='both')
        g=Pmw.Group(self.frame.interior(),tag_text='Requete MySQL',
                    tag_font=self.gui.config.police_i)
        g.pack(side="top",padx = 8, pady = 5,
               expand='yes',anchor='w',fill="both")
        self.FormRequete=Pmw.ScrolledText(g.interior(),
                                          text_padx = 4,text_pady = 4,
                                          usehullsize = 1,hull_width = 600,
                                          hull_height = 150)
        self.FormRequete.pack(side="top",padx=5,pady=5)
        f=Frame(g.interior())
        f.pack(side="top",expand="yes",fill="none",anchor='w',padx=5,pady=5)
        e=Pmw.EntryField(f,labelpos='w',label_text='Requte depuis un fichier : ',
                         entry_textvariable=self.fichier)
        e._entryFieldEntry.grid(sticky ='w')
        e._entryFieldEntry.config(width=30)
        e.pack(side="left",fill="none",expand="no",padx=3,pady=3,anchor='w')
        b=Button(f, text='...', command=self.getFichier)
        b.pack(side='left',fill='none',expand='no',padx=10,pady=5,anchor='w')

    def getFichier(self):
        file=tkFileDialog.askopenfilename()
        if file==():
            pass
	else:
	    self.fichier.set(file)
	    fid=open(self.fichier.get(),'r')
	    self.FormRequete.settext(str(fid.read()))

    def Fermer(self):
        """
        Ferme l'onglet de MySQL
        """
        if self.fullwindow:
            self.top.destroy()
        else:
            self.gui.Fermer()

    def Vider(self):
        """
        Vide le formulaire
        """
        self.FormRequete.settext('')
        self.fichier.set('')
        if self.label is not None:
            self.label.destroy()
            self.label=None
        if self.text is not None:
            self.text.destroy()
            self.text=None
    
    def Executer(self):
        """
        Execute la requete
        """
        if self.label is not None:
            self.label.destroy()
            self.label=None
        if self.text is not None:
            self.text.destroy()
            self.text=None
        try:
            self.connect()
            res=self.cursor.execute(self.FormRequete.get().encode('latin-1'))
            res=self.cursor.fetchall()
            nbres=self.cursor.rowcount
            self.label=Label(self.win.pane("resultat"),
                    text=str(self.connexion.affected_rows())+\
                    " enregistrement affects\n",
                    anchor='n', justify='center',)
            self.label.pack(side="top",expand="no",fill="x")
            text=''
            if nbres==0:
                pass
            else:
                for item in range(nbres):
                    text+='--------------------------\n'
                    text+=tools.printlistes(res[item])
                    text+='--------------------------\n'
                self.text=Pmw.ScrolledText(self.win.pane('resultat'),text_padx = 0,
                                   text_wrap='word',text_bg='#DDDDDD',
                                   text_pady = 4,text_relief='flat')
                self.text.pack(side="top",padx=0,pady=5,fill='both',expand='yes')
                self.text.settext(tools.filtrehtml(text))        
            self.deconnect()  
        except MySQLdb.Error, e:
            msg="Erreur %d: %s" % (e.args[0], e.args[1])
            msg=re.sub(';','\n',msg)
            dialog.Errormsg(self.top,msg).show()
       
        
    

#========================================================================================= 
# Main
#-----------------------------------------------------------------------------------------
if __name__ == '__main__':
    import Tkinter
    import config

    root=Tkinter.Tk()    
    conf=config.Config("./config.xml",root,None)
    mysql_test=Mysql(conf)
    print 'objet cr'
    mysql_test.IsOn()
    print 'MySQL is on'
    mysql_test.DbIsOn()
    print 'Db is on'
    mysql_test.connect()
    print 'connect'
    mysql_test.deconnect()
    print 'deconnect'
    
