# -*- coding: iso8859-1 -*-
#=========================================================================================
#                                                                         
#   This program is a cardware software; you can redistribute it and/or modify
#   it under the terms of the Creative Common Licence defined on :
#   http://creativecommons.org/licenses/by-nc-sa/2.0/fr/
#
#   (CC) 2005 by Ronan BOISARD <ronan_boisard@yahoo.com>
#
#=========================================================================================
#                                                                         
#   Ce programme est un cardware, vous pouvez l'utiliser, le modifier et le redistribuer
#   sous les condition dfinies dans la licence Creative Common dont les condition sont
#   dcritent sur
#   http://creativecommons.org/licenses/by-nc-sa/2.0/fr/
#
#   (CC) 2005 by Ronan BOISARD <ronan_boisard@yahoo.com>
#
#=========================================================================================

#=========================================================================================
"""
    Classe destine a la gestion du 'splashscreen' d'accueil du logiciel
"""
#-----------------------------------------------------------------------------------------
__version__ ='0.4'
__author__='Ronan BOISARD <ronan_boisard@yahoo.com>'
#=========================================================================================

from Tkinter import *
try:
     from PIL import Image, ImageTk
except ImportError:
     import Image, ImageTk
import Pmw

class SplashScreen(Toplevel):
    """
    Classe destine a la gestion du 'splashscreen' d'accueil du logiciel
    """
    def __init__(self, master,width,height,image=None, timeout=1000):
        """
        Initialisation du splash screen

        Inputs:
            master  :   TKinter root de l'application
            image   :   Image a afficher
            timeout :   Dure de l'affichage
            height  :   hauter de la fenetre
            width   :   largeur de la fenetre
        """
        Toplevel.__init__(self, master, relief='raised', borderwidth=5)
        self.main = master
        self.width=width
        self.height=height
        self.msg=StringVar()
        self.msg.set("Chargement de l'application...")
        if self.main.master != None:
            self.main.master.withdraw()
        self.main.withdraw()
        self.overrideredirect(1)
        im = Image.open(image)
        self.image = ImageTk.PhotoImage(im)
        self.after_idle(self.centerOnScreen)
        self.update()
        if timeout is not None:
            self.after(timeout, self.destroy)
        
    def centerOnScreen(self):
        """
        Utilis pour centrer l'image a l'cran
        """
        self.update_idletasks()
        height, width = self.height,self.width
        xmax = self.main.winfo_screenwidth()
        ymax = self.main.winfo_screenheight()
        x0 = self.x0 = (xmax - width) / 2
        y0 = self.y0 = (ymax - height)/2
        self.geometry("%dx%d+%d+%d" % (width,height,x0, y0))
        self.createWidgets()

    def createWidgets(self):
	"""
        Creation de l'ensemble des widget a afficher: image, text...
	"""
	
        self.canvas = Canvas(self, height=self.height-100, width=self.width)
        self.canvas.create_image(0,0, anchor=NW, image=self.image)
        self.canvas.pack(side='top',fill='none',expand='no')
        self.canvas.bind("<ButtonPress-1>",self.destroy)
        text="This program is a cardware software; you can redistribute it "
        text+="and/or modify it under the terms of the Creative Common Licence "
        text+="defined on :\n"
        text+="http://creativecommons.org/licenses/by-nc-sa/2.0/fr/\n"
        text+="(CC) 2005 by Ronan BOISARD <ronan_boisard@yahoo.com>"
        self.t=Text(self, font=('Helvetica',12,'normal'),wrap='word',height=4,relief='flat')
        self.t.pack(side="top",expand="no",fill="x")
        self.t.insert(END,text)
        self.t.bind("<ButtonPress-1>",self.destroy)
        self.msg=StringVar()
        self.l=Label(self,textvariable=self.msg, font=('Helvetica',12,'bold'),anchor='n', justify='left')
        self.l.pack(side="left",expand="yes",fill="both")

    def destroy(self,event='none'):
        """
        Detruit le splashscreen
        """
        self.main.update()
        self.main.deiconify()
        self.withdraw()
